@@ -15,16 +15,16 @@
 #include "luatex_svnversion.h"
 
 static const char _svn_version[] =
-    "$Id: luatex.c 3637 2010-04-20 13:44:56Z taco $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.60.1/source/texk/web2c/luatexdir/luatex.c $";
+    "$Id: luatex.c 3730 2010-07-01 07:12:12Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.60.2/source/texk/web2c/luatexdir/luatex.c $";
 
 #define TeX
 
 int luatex_svn = luatex_svn_revision;
 int luatex_version = 60;        /* \.{\\luatexversion}  */
-int luatex_revision = '1';      /* \.{\\luatexrevision}  */
+int luatex_revision = '2';      /* \.{\\luatexrevision}  */
 int luatex_date_info = -extra_version_info;     /* the compile date is negated */
-const char *luatex_version_string = "beta-0.60.1";
+const char *luatex_version_string = "beta-0.60.2";
 const char *engine_name = "luatex";     /* the name of this engine */
 
 #include <kpathsea/c-ctype.h>
@@ -33,6 +33,10 @@ const char *engine_name = "luatex";     /* the name of this engine */
 #include <kpathsea/variable.h>
 #include <kpathsea/absolute.h>
 #include <kpathsea/recorder.h>
+#ifdef WIN32
+#include <kpathsea/concatn.h>
+#endif
+
 
 #include <time.h>               /* For `struct tm'.  */
 #if defined (HAVE_SYS_TIME_H)
@@ -347,6 +351,36 @@ int shell_cmd_is_allowed(const char **cmd, char **safecmd, char **cmdname)
             *d++ = QUOTE;
         }
         *d = '\0';
+#ifdef WIN32
+        {
+          char *p, *q, *r;
+          p = *safecmd;
+          if (!(IS_DIR_SEP (p[0]) && IS_DIR_SEP (p[1])) &&
+              !(p[1] == ':' && IS_DIR_SEP (p[2]))) { 
+            p = (char *) kpse_var_value ("SELFAUTOLOC");
+            if (p) {
+              r = *safecmd;
+              while (*r && !Isspace(*r))
+                r++;
+              if (*r == '\0')
+                q = concatn ("\"", p, "/", *safecmd, "\"", NULL);
+              else {
+                *r = '\0';
+                r++;
+                while (*r && Isspace(*r))
+                  r++;
+                if (*r)
+                  q = concatn ("\"", p, "/", *safecmd, "\" ", r, NULL);
+                else
+                  q = concatn ("\"", p, "/", *safecmd, "\"", NULL);
+              }
+              free (p);
+              free (*safecmd);
+              *safecmd = q;
+            }
+          }
+        }
+#endif
     }
 
     return allow;
@@ -925,7 +959,7 @@ calledit(packedASCIIcode * filename,
     *temp = 0;
 
     /* Execute the command.  */
-#ifdef WIN32
+#ifdef __MINGW32__
     /* Win32 reimplementation of the system() command
        provides opportunity to call it asynchronously */
     if (win32_system(command, true) != 0)
@@ -952,7 +986,7 @@ calledit(packedASCIIcode * filename,
 /* This macro is always invoked as a statement.  It assumes a variable
    `temp'.  */
 
-#  define SWAP(x, y) temp = (x); (x) = (y); (y) = temp
+#  define SWAP(x, y) do { temp = x; x = y; y = temp; } while (0)
 
 
 /* Make the NITEMS items pointed at by P, each of size SIZE, be the
@@ -961,9 +995,10 @@ calledit(packedASCIIcode * filename,
 void swap_items(char *pp, int nitems, int size)
 {
     char temp;
-    char *q = xmalloc(nitems*size);
+    unsigned total = nitems*size;
+    char *q = xmalloc(total);
     char *p = q;
-    memcpy(p,pp,nitems*size);
+    memcpy(p,pp,total);
     /* Since `size' does not change, we can write a while loop for each
        case, and avoid testing `size' for each time.  */
     switch (size) {
@@ -1027,7 +1062,8 @@ void swap_items(char *pp, int nitems, int size)
     default:
         FATAL1("Can't swap a %d-byte item for (un)dumping", size);
     }
-    xfree(q);
+    memcpy(pp,q,total);
+    xfree(q); 
 }
 #endif                          /* not WORDS_BIGENDIAN and not NO_DUMP_SHARE */
 
